// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/dot/siena.html
//
//  Author: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 1998-1999 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: sender.cc,v 1.2 2002/11/22 20:57:46 carzanig Exp $
//
#include <siena.h>

#include <iostream>
#include <exception>

using namespace std;

#include <siena/Socket.h>

int main (int argc, char *argv[])
{
    Event e;
    Filter f;
    Request r;
    iostream *s = NULL;
    bool quiet = false;

    if (argc > 1 && string(argv[1])==string("-quiet"))
	quiet = true;

    try {
	cin >> r;
	Request::const_iterator ri;
	if ((ri = r.find("to")) == r.end()) {
	    cerr << "missing `to' SENP header" << endl;
	    return 1;
	}
	
	URI u((*ri).second.string_value());
	
	if ((ri = r.find(SENP::Method)) == r.end()) {
	    cerr << "missing `method' SENP header" << endl;
	    return 1;
	}
	if ((*ri).second.type() != Siena_string) {
	    cerr << "non-string `method' header" << endl;
	    return 1;
	}

	const string & method = (*ri).second.string_value();

	s = u.open_connection();
	if (method==SENP::PUB) {
	    cin >> e;
	    *s << r << e << endl << flush;
	} else if (method == SENP::SUB || method == SENP::UNS 
		   || method == SENP::ADV || method == SENP::UNA) {
	    cin >> f;
	    *s << r << f << endl << flush;
	} else {
	    *s << r << endl << flush;
	}
	string reply;
	if (quiet) {
	    if (!getline(*s, reply)) {
		delete(s);
		return 1;
	    }
	    delete(s);
	    const char *cp = reply.c_str();
	    return *cp++ != '2' || *cp++ != '0' || *cp != '0';
	} else {
	    while(getline(*s, reply))
		cout << reply << endl;
	    delete(s);
	    return 0;
	}
    } catch (exception &ex) {
	cerr << "error: " << ex.what() << endl;
	if (s != NULL) delete(s);
	//
	// yeah, whatever!
	//
	return -1;
    }
    return 0;
}
